package org.jutils.collection;
import java.util.*;

/**
 * <code></code>
 * @author: M.Winkler, doubleSlash GmbH
 */
public class GenericIterator implements Iterator {
   private Object[] data_ = null;
   private int index_ = 0;

   public final static java.util.Iterator EMPTY_ITERATOR = new GenericIterator(null);



   /**
    * GenericObjectIterator constructor comment.
    */
   public GenericIterator(Object[] data) {
      super();
      data_ = data;
   }



   /**
    * hasNext method comment.
    */
   public boolean hasNext() {
      boolean returnValue = false;
      if (data_ != null) returnValue =index_<data_.length ;
      return returnValue ;
   }



   /**
    * Restart the iterator from the start. This is NOT a default behavior of the iterator,
    * and also is NOT a behavior which is defined by <code>java.util.Iterator</code>,
    * but its a nice feature, if you know what you do ;).
    */
    public void restart () {
      index_ = 0;
    }



   /**
    * next method comment.
    */
   public Object next() {

      // check wether we still have some elements.
      if (!hasNext())
         throw new NoSuchElementException ();

      // init method
      Object returnValue = null;

      // define return value
      returnValue = data_[index_];

      // raise index
      index_++;

      // thats it
      return returnValue;
   }



   /**
    * remove method comment.
    */
   public void remove() {
      throw new UnsupportedOperationException("Cannot remove elements from genericIterator.");
   }



   /**
    * Returns a String that represents the value of this object.
    * @return a string representation of the receiver
    */
   public String toString() {
      return data_.toString() ;
   }



  /**
  * Tests this class
  */
  public static void main (String [] args) {
   System.out.println("Testing GenericIterator ...");
   int errorCount = 0 ;
   GenericIterator iterator = new GenericIterator (null);

   // check wether or not the null iterator does return something (which would be a fault)
   System.out.println ( "Testing null iterator ..." );
   if (iterator.hasNext()) {
     System.out.println("ERROR --- empty iterator has elements!");
     errorCount++;
   }

   // check full iterator
   System.out.println ( "Testing number of elements in full iterator ..." );
   iterator = new GenericIterator (new String [] {"Alpha","Beta","Gamma","Delta"});
   int counter = 0 ;
   while (iterator.hasNext()) {
     counter ++;
     iterator.next() ;
   }

   if (counter  != 4) {
      System.out.println("ERROR --- iterator does not contain the correct number of elements!");
      errorCount++;
   }

   // check restart
   System.out.println("Testing restart ...");
   iterator.restart();
   if (!iterator.next().equals("Alpha")) {
      System.out.println("ERROR --- restart did not work!");
      errorCount++;
   }


   // finish
   System.out.println("--------------------------------");
   System.out.println(""+errorCount+" errors detected.");
   System.out.println("--------------------------------");
  }
}
