package org.jutils.collection;

/**  This class creates a "Long" hashtable. Now it is possible to insert "long" Values.
Most parts are from the original hashtable-class so if there are some problems, but I think
there are none, look in java.util.hashtable!!!! **/
import java.util.*;
import java.io.*;

public class LongHashtable {
    // private transient Set entrySet = null;
    String toStringResult = null;
    // private transient int modCount = 0;
    private transient int count_;
    private transient int linearCount_ = -1;
    private int threshold_;
    private static class Entry {
        int hash_;
        long key_;
        Object value_;
        Entry next_;
        protected Entry(int hash, long key, Object value, Entry next) {
            hash_ = hash;
            key_ = key;
            value_ = value;
            next_ = next;
        }
        protected Object clone() {
            return new Entry(
                hash_, 
                key_, 
                value_, 
                (next_ == null ? null : (Entry) next_.clone())); 
        }

        // Map.Entry Ops

        public long getKey() {
            return key_;
        }
        public Object getValue() {
            return value_;
        }
        public Object setValue(Object value) {
            if (value == null)
                throw new NullPointerException();
            Object oldValue = value_;
            value_ = value;
            return oldValue;
        }
        public boolean equals(Object o) {
            boolean returnValue = false;
            try {
                Entry e = (Entry) o;
                returnValue = (e == this);
            } catch (ClassCastException cce) {
                returnValue = false;
            } catch (NullPointerException npe) {
                returnValue = false;
            }
            return returnValue;
        }
        public int hashCode() {
            return hash_ ^ (value_ == null ? 0 : value_.hashCode());
        }
        public String toString() {
            return "" + key_ + "=" + value_.toString();
        }
    }
    private float loadFactor_;
    private Entry table_[];



    /**
     * LongHashtable constructor comment.
     */
    public LongHashtable() {
        this(101, 0.75f);
    }



    /** Construktor which creates a Hashtable with a specified capacity and a loadFactor **/
    
    public LongHashtable(int initialCapacity, float loadFactor) {
        if (initialCapacity < 0)
            throw new IllegalArgumentException("Illegal Capacity: " + initialCapacity);
        if (loadFactor <= 0)
            throw new IllegalArgumentException("Illegal Load: " + loadFactor);
        if (initialCapacity == 0)
            initialCapacity = 1;
        this.loadFactor_ = loadFactor;
        table_ = new Entry[initialCapacity];
        threshold_ = (int) (initialCapacity * loadFactor);
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     */
    public int capacity() {
        return table_.length; 
    }



    /**
     * Insert the method's description here.
     * Creation date: (02.10.2000 11:50:30)
     */
    public void clear() {
        for (int index = 0; index < table_.length; index++) {
            table_[index] = null;
        }
        pInvalidateCachedValues();
    }



    /** Thi method returns true for an existing key **/
    
    
    public boolean containsKey(long key2) {
        Object value = get(key2);
        if (value != null)
            return true;
        return false;
    }



    /**
    * Returns the value from the spezified key.
    * @author Christian Brugger, in corporation with M.Winkler
    **/
    
    public synchronized Object get(long key) {
        Entry tab[] = table_;
        int hash = pKeyToHash(key);
        int index = hash % tab.length;
        for (Entry e = tab[index]; e != null; e = e.next_) {
            if ((e.hash_ == hash) && (e.key_ == key)) {
                return e.value_;
            }
        }
        return null;
    }



    /**
    
    returns an array of "Objects" which contains all values of the Hashtable
    
     **/
    public Object[] getValuesArray() {
        Object returnValue[] = new Object[size()];
        long zaehler = 0;
        for (int i = 0; i < size(); i++) {
            if (get(zaehler) == null) {
                zaehler++;
                i--;
            } else {
                returnValue[i] = get(zaehler);
                zaehler++;
            }
        }
        return returnValue;
    }



    /** returns the number of entries **/
    
    
    public int linearCount() {
        if (linearCount_ < 0) {
            linearCount_ = 0;
            for (int index = 0; index < table_.length; index++) {
                if (table_[index] != null)
                    linearCount_++;
            }
        }
        return linearCount_;
    }



    /**
    The main - method is for test if it is working
    **/
    public static void main(String[] args) {
        long[] testArray = {4, 3, 10, 0, 1, 9};
        Object[] testObject = new Object[6];
        testObject[0] = "hallo ";
        testObject[1] = "mein";
        testObject[2] = "Hashtable";
        testObject[3] = "einwandfrei";
        testObject[4] = "gell";
        testObject[5] = "funktioniert";
        int errorCount = 0;
        LongHashtable test = null;
        try {
            test = new LongHashtable();
            long maxCount = testArray.length;
            for (int i = 0; i < maxCount; i++) {
                test.put(testArray[i], testObject[i]);
            }
            System.out.println("---------------------------");
            System.out.println("Statistics :");
            System.out.println("---------------------------");
            System.out.println(test.size() + " elements in hashtable");
            System.out.println(test.capacity() + " linear elements possible");
            System.out.println(test.linearCount() + " linear elements present");
            System.out.println((int) (test.linearCount() * 100 / test.capacity()) + " % linear load.");
            System.out.println((int) (test.size() * 100 / test.capacity()) + " % total load.");
            System.out.println("---------------------------");
            System.out.println("Testing count ...");
            if (test.size() != testObject.length) {
                System.out.println("!!! Elements in table do not match the amount of elements put into it! count is " + test.size());
                errorCount++;
            }
            System.out.println("Testing getValuesArray() ...");
            Object[] testValueToArray = test.getValuesArray();
            if (testValueToArray.length != testObject.length) {
                System.out.println("!!! Size of returned array does not match! Size = " + testValueToArray.length + ", should be " + testObject.length);
                errorCount++;
            }
            List testObjects = CollectionHelper.toList(testObject, null);
            List returnedObjects = CollectionHelper.toList(testValueToArray, null);
            List cut = CollectionHelper.createCut(testObjects, returnedObjects, null);
            if (cut.size() != testObject.length) {
                System.out.println("!!! Elements do not match !");
                errorCount++;
            }
            for (int i = 0; i < maxCount; i++) {
                Object o = test.get(testArray[i]);
                if (o != null) {
                    if (!o.equals(testObject[i])) {
                        System.out.println("Failure elements not equal : " + o + " should be " + testObject[i]);
                        errorCount++;
                    }
                }
                else {
                    System.out.println("Table does return null with key : " + testObject[i]);
                    errorCount++;
                }
            }
            System.out.println("Testing for existing key...");
            if (!test.containsKey(3)) {
                System.out.println("!!! Existing key 3 not found!");
                errorCount++;
            }
            System.out.println("Testing for non-existing positive key...");
            if (test.containsKey(2)) {
                System.out.println("!!! Non-existing key 2 found!");
                errorCount++;
            }
            System.out.println("Testing for non-existing negative key...");
            if (test.containsKey(-10000)) {
                System.out.println("!!! Non-existing key -10000 found!");
                errorCount++;
            }
            System.out.println("Testing for correct replacement...");
            test.put(testArray[0], testObject[testObject.length - 1]);
            if (test.get(testArray[0]) != testObject[testObject.length - 1]) {
                System.out.println("!!! Does not replace existing elements!");
                errorCount++;
            }
            System.out.println("Test putting null values ...");
            try {
                test.put(testArray[0], null);
                System.out.println("!!! Does accept null as a value!");
                errorCount++;
            }
            catch (NullPointerException npe) {
                // Everything is okay.
            }
            System.out.println("Test putting with negative keys ...");
            test.put(-1, testObject);
            if (!test.containsKey(-1) || (test.get(-1) != testObject)) {
                System.out.println("!!! Did not accept a negative key!");
                errorCount++;
            }
            System.out.println("Testing overloading ...");
            for (int index = 100; index < 10000; index++) {
                Object value = String.valueOf(index);
                long key = index;
                test.put(key, value);
            }
            for (int index = 100; index < 10000; index++) {
                long key = index;
                String value = (String) test.get(key);
                if (value != null) {
                    long returnedLong = Long.parseLong(value);
                    if (returnedLong != key) {
                        System.out.println("!!! Wrong value returnded :" + value + " ahould be " + key);
                        errorCount++;
                    }
                }
                else {
                    System.out.println("!!! Null value returnded : key " + key + " ahould return" + key);
                    errorCount++;
                }
            }
            int expectedCount = (9900 + testObject.length + 1);
            if (test.size() != expectedCount) {
                System.out.println("!!! Count mismatch :" + test.size() + " should be " + expectedCount);
            }
            System.out.println("Checking toString ");
            String toString1 = test.toString();
            String toString2 = test.toString();
            if (toString1 != toString2) {
                System.out.println("!!! toString did calculate twice!");
                errorCount++;
            }
            test.remove(Long.MIN_VALUE);
            toString2 = test.toString();
            if (toString1 != toString2) {
                System.out.println("!!! toString did an invalid remove detect as a change!");
                errorCount++;
            }
            System.out.println("---------------------------");
            System.out.println("Statistics :");
            System.out.println("---------------------------");
            System.out.println(test.size() + " elements in hashtable");
            System.out.println(test.capacity() + " linear elements possible");
            System.out.println(test.linearCount() + " linear elements present");
            System.out.println((int) (test.linearCount() * 100 / test.capacity()) + " % linear load.");
            System.out.println((int) (test.size() * 100 / test.capacity()) + " % total load.");
            System.out.println("---------------------------");
        }
        catch (NullPointerException e) {
            System.out.println(e.toString());
        }
        finally {
            System.out.println();
            System.out.println(errorCount + " errors found.");
        }
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     */
    private void pInvalidateCachedValues() {
        toStringResult = null;
        linearCount_ = -1;
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return int
     * @param key long
     */
    private int pKeyToHash(long key) {
        int returnValue;
        returnValue = (int) (key % Integer.MAX_VALUE);
        if (returnValue < 0)
            returnValue = returnValue * -1;
        return returnValue;
    }



    /**
    * inserts an value into the hashtable which is specified with a key
    **/
    public synchronized Object put(long key, Object value) {
        // Make sure the value is not null
        if (value == null) {
            throw new NullPointerException();
        }
    
        // Makes sure the key is not already in the hashtable.
        Entry tab[] = table_;
        int hash = pKeyToHash(key);
        int index = hash % tab.length;
        for (Entry e = tab[index]; e != null; e = e.next_) {
            if ((e.hash_ == hash) && (e.key_ == key)) {
                Object old = e.value_;
                e.value_ = value;
                pInvalidateCachedValues();
                return old;
            }
        }
        //     modCount++;
        if (count_ >= threshold_) {
            // Rehash the table if the threshold is exceeded
            rehash();
            tab = table_;
            index = hash % tab.length;
        }
    
        // Creates the new entry.
        Entry e = new Entry(hash, key, value, tab[index]);
        tab[index] = e;
        count_++;
        return null;
    }



    protected void rehash() {
        int oldCapacity = table_.length; 
        Entry oldMap[] = table_;
        int newCapacity = oldCapacity * 2 + 1;
        Entry newMap[] = new Entry[newCapacity];
        //     modCount++;
        threshold_ = (int) (newCapacity * loadFactor_);
        table_ = newMap;
        for (int i = oldCapacity; i-- > 0;) {
            for (Entry old = oldMap[i]; old != null;) {
                Entry e = old;
                old = old.next_;
                int index = e.hash_ % newCapacity;
                e.next_ = newMap[index];
                newMap[index] = e;
            }
        }
        pInvalidateCachedValues();
    }



    public boolean remove(long key) {
        int hash = pKeyToHash(key);
        int index = hash % table_.length;
        for (Entry e = table_[index], prev = null; e != null; prev = e, e = e.next_) {
            if ((e.hash_ == hash) && (e.key_ == key)) {
                //             modCount++;
                if (prev != null)
                    prev.next_ = e.next_;
                else
                    table_[index] = e.next_;
                count_--;
                e.value_ = null;
                pInvalidateCachedValues();
                return true;
            }
        }
        return false;
    }



    /** returns the number of entries **/
    
    
    public int size() {
        return count_;
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.String
     */
    public String toString() {
        if (toStringResult == null) {
            StringBuffer returnValue = new StringBuffer("{");
            boolean alreadyFoundSomething = false;
            for (int index = 0; index < table_.length; index++) {
                Entry e = table_[index];
                if (e != null) {
                    if (alreadyFoundSomething) {
                        returnValue.append(org.jutils.CharConstants.COMMA);
                    }
                    else
                        alreadyFoundSomething = true;
                    returnValue.append(e.toString());
                }
            }
            returnValue.append("}");
            toStringResult = returnValue.toString();
        }
        return toStringResult;
    }



    /** Construktor which creates a Hashtable with a specified capacity and a loadFactor **/
    
    public LongHashtable(int initialCapacity) {
        this(initialCapacity, 0.75f);
    }
}

