package org.jutils.io;

import java.io.*;
import javax.swing.filechooser.*;
import org.jutils.*;
/**
 * Implements a generic file filter that can adapt for different file names/appendixes.
 * @author: M.Winkler, doubleSlash GmbH
 */
public class GenericFileFilter extends javax.swing.filechooser.FileFilter implements java.io.FileFilter {
    private String fileName_ = null;
    private String fileExt_ = null;
    private String fileDesc_ = null;



/**
 * GenericFileFilter constructor comment.
 */
private GenericFileFilter() {
    super();
}



/**
 * Creates a new GenericFileFilter.
 * @author Michael Winkler, doubleSlash GmbH
 * 
 * @param fileName java.lang.String A file name that may be accepted. may be null to indicate that the file name is not a criteria.
 * @param extension java.lang.String A file Extension to be accepted, withoput any containing periods. Also may be null.
 * @param description java.lang.String A description of the files accepted by this file filter.
 */
public GenericFileFilter(String completeFileNameOrExtension) {
    fileName_ = null;
    fileExt_ = completeFileNameOrExtension;
    fileDesc_ = "";
}



/**
 * Creates a new GenericFileFilter.
 * @author Michael Winkler, doubleSlash GmbH
 * 
 * @param extension java.lang.String A file Extension to be accepted, withoput any containing periods. Also may be null.
 * @param description java.lang.String A description of the files accepted by this file filter.
 */
public GenericFileFilter(String extension, String description) {
    fileName_ = null;
    fileExt_ = extension;
    fileDesc_ = description;
}



/**
 * Creates a new GenericFileFilter.
 * @author Michael Winkler, doubleSlash GmbH
 * 
 * @param fileName java.lang.String A file name that may be accepted. may be null to indicate that the file name is not a criteria.
 * @param extension java.lang.String A file Extension to be accepted, withoput any containing periods. Also may be null.
 * @param description java.lang.String A description of the files accepted by this file filter.
 */
public GenericFileFilter(String fileName, String extension, String description) {
    fileName_ = fileName;
    fileExt_ = extension;
    fileDesc_ = description;
}



/**
 * accept method comment.
 */
public boolean accept(java.io.File f) {
    boolean returnValue = false;

    // check if its a file
    returnValue = false;
    String fileName = f.getName();

    // check its file extension (but only if it really is a file
    if (!f.isDirectory()) {
        if (fileExt_ != null)
            returnValue = fileName.endsWith(fileExt_);

        // check its file name (but only if it's not yet accepted)
        if (!returnValue && (fileName_ != null)) {

            // separate the file name from the extension
            int lastPointIndex = fileName.lastIndexOf(CharConstants.PERIOD_STRING);

            // is there a period in the string?!?
            if (lastPointIndex > -1) {

                // is there somnething before the period?
                if (lastPointIndex > 0) {
                    String first = fileName.substring(0, lastPointIndex);
                    returnValue = fileName.equals(first);
                }
                else {
                    // there is nothing before the period; 
                    // nevertheless we have to handle this.
                    fileName = "";
                }
            }
            else {
                // if there's no period in the string, compare filenames directly
                returnValue = fileName.equals(fileName_);
            }
        }
    }
    return returnValue;
}



/**
 * getDescription method comment.
 */
public String getDescription() {
    return fileDesc_;
}



/**
 * Starts the application.
 * @param args an array of command-line arguments
 */
public static void main(java.lang.String[] args) {

    // Init
    // --------------------------------------------------------------
    System.out.println("Testing GenericFileFilter ...");
    int errorCount = 0;
    GenericFileFilter filter = new GenericFileFilter("anyName", ".bla", "Dont watch this");

    // Test perfect file
    File file = new File(File.pathSeparator + "anyName.bla ");
    if (!filter.accept(file)) {
        errorCount++;
        System.err.println("Cannot match perfect file ...");
    }

    // testing file extension only
    file = new File(".bla");
    if (!filter.accept(file)) {
        errorCount++;
        System.err.println("Cannot match extension - only file ...");
    }

    // testing wrong name, but correct file extension
    file = new File("someFile.bla");
    if (!filter.accept(file)) {
        errorCount++;
        System.err.println("Cannot match file with correct extension !");
    }

    // testing correct name, but wrong file extension
    file = new File("anyFile.stub");
    if (!filter.accept(file)) {
        errorCount++;
        System.err.println("Cannot match file with correct name!");
    }

    // stats
    System.out.println("" + errorCount + " errors detected.");
}
}

