package org.jutils.net;
/*------------------------------------------------------------------------------
Name:      IPHelper.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   The named logging instance which delegates the logging task
           to all registered logging devices in this object.
------------------------------------------------------------------------------*/


import org.jutils.text.StackTrace;


/*******************************************************************************
*******************************************************************************/
public class IPHelper
{
   /****************************************************************************
    * This method converts a valid IPv4 address string of the format 
    * "%d.%d.%d.%d" to an nativ long value. The result of this method is 
    * allways a positiv value.
    * 
    * @param      ipString
    *             The string with the IP to convert
    * @return     The converted value
    * @exception  java.lang.IllegalArgumentException
    *             This exception is thrown if the given String has no valid 
    *             format.
    ***************************************************************************/
   public static long ipv4GetLong(String ipString)
      throws IllegalArgumentException
   {
      try {
         int delimPos3 = ipString.indexOf(".");
         String part4 = ipString.substring(0, delimPos3);
         long val4 = Long.parseLong(part4);
         if (val4 < 0 || val4 > 255)
            throw new IllegalArgumentException("Value '" + val4 + "' out of range");

         int delimPos2 = ipString.indexOf(".", delimPos3 + 1);
         String part3 = ipString.substring(delimPos3 + 1, delimPos2);
         long val3 = Long.parseLong(part3);
         if (val3 < 0 || val3 > 255)
            throw new IllegalArgumentException("Value '" + val3 + "' out of range");

         int delimPos1 = ipString.indexOf(".", delimPos2 + 1);
         String part2 = ipString.substring(delimPos2 + 1, delimPos1);
         long val2 = Long.parseLong(part2);
         if (val2 < 0 || val2 > 255)
            throw new IllegalArgumentException("Value '" + val2 + "' out of range");

         String part1 = ipString.substring(delimPos1 + 1, ipString.length());
         long val1 = Long.parseLong(part1);
         if (val1 < 0 || val1 > 255)
            throw new IllegalArgumentException("Value '" + val1 + "' out of range");

         return (val4*256*256*256 + val3*256*256 + val2*256 + val1);
      }
      catch (Exception e) {
         throw new IllegalArgumentException("The given parameter '" + ipString + 
            "' is no valid IPv4 address.\nREASON STACKTRACE:\n" + 
            StackTrace.getStackTrace(e));
      }
   }


   /****************************************************************************
    * This method converts a valid IPv4 address string of the format 
    * "%d.%d.%d.%d" to an nativ int value. The result of this method may 
    * be a negative value.
    * 
    * @param      ipString
    *             The string with the IP to convert
    * @return     The converted value
    * @exception  java.lang.IllegalArgumentException
    *             This exception is thrown if the given String has no valid 
    *             format.
    ***************************************************************************/
   public static int ipv4GetInt(String ipString)
   {
      return (int)ipv4GetLong(ipString);
   }


   /****************************************************************************
    * Test suitcase
    ***************************************************************************/
   public static void main(String[] args)
   {
      System.out.println();
      System.out.println("LONG-Value of '192.168.1.1' is " + ipv4GetLong("192.168.1.1"));
      System.out.println("INT-Value  of '192.168.1.1' is " + ipv4GetInt("192.168.1.1"));
      System.out.println();
   }
}
