package org.jutils.text;
/*------------------------------------------------------------------------------
Name:      LinkExtractor.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   The named logging instance which delegates the logging task
           to all registered logging devices in this object.
------------------------------------------------------------------------------*/


import java.io.*;
import java.net.*;
import javax.swing.text.*;
import javax.swing.text.html.*;


/*******************************************************************************
 * This is a very special helper class for extracting HTML links.
 *
 * @author Wolfgang Kleinertz
 ******************************************************************************/
public class LinkExtractor
{
   /****************************************************************************
    * 
    ***************************************************************************/
   public static void main(String[] args)
   {
      EditorKit kit = new HTMLEditorKit();
      Document doc = kit.createDefaultDocument();

      doc.putProperty("IgnoreCharsetDirective", Boolean.TRUE);
      try {
         Reader rd = getReader(args[0]);

         kit.read(rd, doc, 0);

         ElementIterator it = new ElementIterator(doc);
         javax.swing.text.Element elem;
         while ((elem = it.next()) != null) {
            SimpleAttributeSet s = (SimpleAttributeSet)
            elem.getAttributes().getAttribute(HTML.Tag.A);
            if (s != null) {
               System.out.println(args[0] + " : " + 
                  s.getAttribute(HTML.Attribute.HREF));
            }
         }
      } catch (Exception e) {
         e.printStackTrace();
      }
      System.exit(1);
   }


   /****************************************************************************
    * 
    ***************************************************************************/
   private static Reader getReader(String uri) 
      throws IOException
   {
      if (uri.startsWith("http:")) {
         URLConnection conn = new URL(uri).openConnection();
         return new InputStreamReader(conn.getInputStream());
      } else {
         return new FileReader(uri);
      }
   }
}
